/// CaveGame - A procedural 2D platformer sandbox.
/// Created by Josh O'Leary @ Redacted Software, 2020-2025
/// Contact: josh@redacted.cc
/// Contributors: william@redacted.cc maxi@redacted.cc
/// This work is dedicated to the public domain.

/// @file CreditsWindow.hpp
/// @desc The UI window for the credits.
/// @edit 1/28/2025
/// @auth Josh O'Leary

#pragma once
#include "JUI/Widgets/Window.hpp"
#include "JUI/Widgets/TextRect.hpp"
#include "JUI/Widgets/ListLayout.hpp"

namespace CaveGame::Client
{
    using namespace JUI;

    struct Accreditation {
        unsigned int text_size;
        Color4 text_color;
        std::string content;
    };

    // We currently lack Rich Text, so we'll do this...
    // NOTE: this must be static const, or the program will segfault upon exit.
    static const std::vector<Accreditation> Credits = {
            {50, Colors::White, "CaveGame"},
            {12, Colors::Gray, "Version 3 Alpha, Prerelease-3"},
            {24, Colors::Blues::LightSteelBlue, "A Redacted Software Product"},
            {20, Colors::Yellows::Moccasin, "Josh O'Leary"},
            {14, Colors::Grays::LightSlateGray, "Programming & Design"},
            {20, Colors::Grays::DarkSlateGray, "William Redacted"},
            {14, Colors::Grays::LightSlateGray, "Programming"},
            {20, Colors::Reds::LightCoral, "Ash Tray"},
            {14, Colors::Grays::LightSlateGray, "Art & Design"},
            {18, Colors::White, "Special Thanks"},
            {12, Colors::Grays::LightSlateGray, "Maxine Hayes"},
            {12, Colors::White, "Evan Walter"},
            {12, Colors::Grays::LightSlateGray, "Karl Darling"},
            {12, Colors::White, "bumpylegoman02"},
            {12, Colors::Grays::LightSlateGray, "ConcurrentSquared"},
            {12, Colors::White, "Chase Carlson"},
            {16, Colors::White, "Made Possible By "},
            {14, Colors::Purples::Violet, "Asperger's Syndrome"},
            {16, Colors::Gray, "(c) 2019-2025 redacted.cc"},

    };

    JUI::Window* CreateCreditsWindowWidget(Widget* parent);
}