/// CaveGame - A procedural 2D platformer sandbox.
/// Created by Josh O'Leary @ Redacted Software, 2020-2025
/// Contact: josh@redacted.cc
/// Contributors: william@redacted.cc maxi@redacted.cc
/// This work is dedicated to the public domain.

/// @file MainMenu.hpp
/// @desc The scene for the home screen, aka main menu.
/// @edit 1/28/2025
/// @auth Josh O'Leary

#pragma once

#include <Client/Scene.hpp>
#include <JUI/JUI.hpp>
#include <JUI/Widgets/Scene.hpp>
#include <JUI/Widgets/TextRect.hpp>
#include <JUI/Widgets/ScrollingRect.hpp>
#include "JUI/Widgets/Button.hpp"
#include <JUI/Widgets/TextButton.hpp>

namespace CaveGame::Client
{

    struct SingleplayerSessionInfo {
        bool NewWorld;
    };

    class MainMenu : public Scene {
    public:
        Event<SingleplayerSessionInfo> RequestWorld;
        Event<> RequestQuit;
        Event<> RequestToggleCredits;
        Event<> RequestToggleSettings;

        MainMenu();
        void Update(float elapsed) override;
        void Draw() override;
        void Load() override;
        void Unload() override;

        void BuildWidgets();
        void PassWindowSize(const J3ML::LinearAlgebra::Vector2 &size) override;
        void PassMouseInput(unsigned int, bool) override;
        void PassMouseMovement(const J3ML::LinearAlgebra::Vector2 &pos) override;
    protected:
        JUI::Scene* scene = nullptr;
        JUI::Rect* title = nullptr;
        JUI::Rect* button_group = nullptr;
        JUI::Rect* changelog = nullptr;
        std::shared_ptr<JGL::Texture> bg = nullptr; // TODO: RAII on this

        Vector2 mpos {0,0};
        Vector2 goal {0,0};
    private:
        JUI::TextButton *create_mainmenu_btn(const std::string &content, JUI::Widget *parent);
    };

}