/// CaveGame - A procedural 2D platformer sandbox.
/// Created by Josh O'Leary @ Redacted Software, 2020-2025
/// Contact: josh@redacted.cc
/// Contributors: william@redacted.cc maxi@redacted.cc
/// This work is dedicated to the public domain.

/// @file Particle.hpp
/// @desc Client-side particle engine, still a work-in-progress.
/// @edit 1/28/2025
/// @auth Josh O'Leary

#pragma once
#include <Color4.hpp>
#include <J3ML/LinearAlgebra/Vector3.hpp>
#include <JGL/JGL.h>

namespace CaveGame::Client
{

    struct Particle
    {
        Vector2 position;
        Vector2 velocity;
        Color4 color;
        float size;
        float life; // Remaining life of the particle. If < 0: dead and unused.
        float angle;
        float weight;
        void Update(float elapsed)
        {
            life -= elapsed;
            if (life <= 0)
                return;

            position += velocity * elapsed;
        }

        void Draw()
        {
            JGL::J2D::DrawPoint(color, position, size);
        }
    };

}
