#pragma once

#include <JUI/Widgets/Rect.hpp>
#include <JUI/Widgets/ListLayout.hpp>
#include <JUI/Widgets/TextButton.hpp>

namespace CaveGame::Client
{
    using namespace JUI::UDimLiterals;

    /// CaveGame's pause menu is implemented as a JUI::Rect overlay, parented to the GameSession::HUD().
    class PauseMenuWidget : public JUI::Rect {
    public:
        Event<> OnResumeButtonPressed;
        Event<> OnSettingsButtonPressed;
        Event<> OnQuitButtonPressed;

        /// The default constructor initializes the pause menu, subordinate widgets, and lays out the visual properties of each.
        PauseMenuWidget();

        /// Construct the Pause menu by specifying it's parent widget.
        /// It is safe to assume the parent will **ALWAYS** be the Scene* instance GameSession::hud.
        explicit PauseMenuWidget(Widget* parent);

        bool ObserveMouseInput(JUI::MouseButton btn, bool pressed) override;

        /// Switches the PauseMenu to it's alternate state, which is either "Open" or "Closed".
        void Toggle();

        /// Sets the state of the PauseMenu.
        void SetOpen(bool value);

        /// @return The state of the PauseMenu.
        [[nodiscard]] bool GetOpen() const;

        /// Sets the state of the PauseMenu to "Open".
        void Open();

        /// Sets the state of the Paus
        void Close();

    protected:
        JUI::Rect* button_box;
        JUI::VerticalListLayout* button_list;
        JUI::TextButton* resume_btn;
        JUI::TextButton* settings_btn;
        JUI::TextButton* quit_btn;

        bool is_open;
    private:
    };
}
