/// CaveGame - A procedural 2D platformer sandbox.
/// Created by Josh O'Leary @ Redacted Software, 2020-2025
/// Contact: josh@redacted.cc
/// Contributors: william@redacted.cc maxi@redacted.cc
/// This work is dedicated to the public domain.

/// @file Splash.hpp
/// @desc The start-screen for the game. Displays our logo, and a loading bar.
/// @edit 1/28/2025
/// @auth Josh O'Leary

#pragma once

#include <Client/Scene.hpp>
#include <JGL/JGL.h>
#include <array>

namespace CaveGame::Client
{
    /// The splash screen is responsible for displaying the Redacted Software Logo and then showing the loading bar.
    class Splash : public Scene
    {
    public:
        Splash();
        ~Splash() override;
        void Draw() override;
        void Update(float elapsed) override;
        void Load() override;
        void Unload() override;

        [[nodiscard]] bool SplashComplete() const;
    protected:
        float splash_timer = 1.5f;
        float load_percent = 0.f;
        std::shared_ptr<JGL::Texture> splash = nullptr;

        std::array<JGL::RenderTarget*, 16> column_textures{};
        int column_width = 0;
        std::vector<float> scroll_offsets;
        std::vector<std::vector<char>> matrix;

    protected:
        void ComputeMatrixGlyphTable();
        void ComputeMatrixTextureCache();

        void DrawMatrix();
        void DrawProgressBar();

    };
}