#include <Client/PauseMenu.hpp>

namespace CaveGame::Client
{

    PauseMenuWidget::PauseMenuWidget() {

        this->Size({100_percent, 100_percent}); // Full Screen.
        this->BGColor({32, 32, 32, 128});    // Transparent Gray.
        this->ZIndex(1);

        button_box = new JUI::Rect(this);
        button_box->Size({150_px, 170_px});
        button_box->AnchorPoint({0.f, 1.f});
        button_box->Position({50_px, 100_percent - 50_px});
        button_box->BGColor(Colors::Transparent);
        button_box->BorderWidth(0);

        button_list = new JUI::VerticalListLayout(button_box);
        button_list->PaddingBottom(5_px);


        resume_btn = new JUI::TextButton(button_list);
        resume_btn->Size({100_percent, 50_px});
        resume_btn->CornerRounding(7);
        resume_btn->Center();
        resume_btn->TextSize(24);
        resume_btn->TextColor(Colors::White);
        resume_btn->BaseBGColor(Colors::DarkGray);
        resume_btn->HoveredBGColor(Colors::Blues::LightSteelBlue);
        resume_btn->BGColor(resume_btn->BaseBGColor());
        resume_btn->Content("Resume");

        resume_btn->OnClickEvent += [this] (Vector2 pos, JUI::MouseButton state) {
            OnResumeButtonPressed.Invoke();
        };

        settings_btn = new JUI::TextButton(button_list);
        settings_btn->Size({100_percent, 50_px});
        settings_btn->CornerRounding(7);
        settings_btn->Center();
        settings_btn->TextSize(24);
        settings_btn->TextColor(Colors::White);
        settings_btn->BaseBGColor(Colors::DarkGray);
        settings_btn->HoveredBGColor(Colors::Blues::LightSteelBlue);
        settings_btn->BGColor(settings_btn->BaseBGColor());
        settings_btn->Content("Settings");

        settings_btn->OnClickEvent += [this] (Vector2 pos, JUI::MouseButton state) {
            OnSettingsButtonPressed.Invoke();
        };

        quit_btn = new JUI::TextButton(button_list);
        quit_btn->Size({100_percent, 50_px});
        quit_btn->CornerRounding(7);
        quit_btn->Center();
        quit_btn->TextSize(24);
        quit_btn->TextColor(Colors::White);
        quit_btn->BaseBGColor(Colors::DarkGray);
        quit_btn->HoveredBGColor(Colors::Blues::LightSteelBlue);
        quit_btn->BGColor(quit_btn->BaseBGColor());
        quit_btn->Content("Save & Exit World");

        quit_btn->OnClickEvent += [this] (Vector2 pos, JUI::MouseButton state) {
            OnQuitButtonPressed.Invoke();
        };

        // Default-initialize as closed.
        this->Close();
    }

    PauseMenuWidget::PauseMenuWidget(Widget *parent): PauseMenuWidget() {
        this->Parent(parent);
    }

    bool PauseMenuWidget::ObserveMouseInput(JUI::MouseButton btn, bool pressed) {
        if (is_open)
            return Rect::ObserveMouseInput(btn, pressed);

        return false;
    }

    void PauseMenuWidget::Toggle() {
        SetOpen(!GetOpen());
    }

    void PauseMenuWidget::SetOpen(bool value) {
        is_open = value;
        this->Visible(is_open);
    }

    bool PauseMenuWidget::GetOpen() const { return is_open; }

    void PauseMenuWidget::Open() { SetOpen(true);  }

    void PauseMenuWidget::Close() { SetOpen(false); }
}

