#include <Client/TileTool.hpp>
#include <Core/Loggers.hpp>

CaveGame::Client::TileTool::TileTool(JUI::Widget *parent) : JUI::Window(parent)
{
    this->Title(tool_title);
    this->MinSize({200, 400});
    this->Size({200, 400, 0, 0});
    this->Visible(false);

    auto* column_layout = new HorizontalListLayout(this->ViewportInstance());

    auto* col_left = new Rect(column_layout);
    col_left->BGColor({64, 64, 64, 128});
    col_left->Size({0, 0, 0.35f, 1.f});

    auto* left_row_layout = new VerticalListLayout(col_left);

    tool_size_label = new TextRect(left_row_layout);
    tool_size_label->Size({0, row_height, 1, 0});
    tool_size_label->Content("Size: 8");
    tool_size_label->BGColor(Colors::Transparent);
    //tool_size_label->LayoutOrder(1);

    auto* tool_percent_label = new TextRect(left_row_layout);
    tool_percent_label->Size({0, row_height, 1, 0});
    tool_percent_label->Content("Percent: 100%");
    tool_percent_label->BGColor(Colors::Transparent);

    auto* tile_sim_label = new TextRect(left_row_layout);
    tile_sim_label->Size({0, row_height, 1, 0});
    tile_sim_label->Content("Pause Tile Sim:");
    tile_sim_label->BGColor(Colors::Transparent);

    auto* col_right = new Rect(column_layout);
    col_right->BGColor({128, 128, 128, 128});
    col_right->Size({0, 0, 0.65f, 1.f});

    auto* right_row_layout = new VerticalListLayout(col_right);

    brush_size_slider = new Slider(right_row_layout);

    brush_size_slider->Size({0, row_height, 1, 0});
    brush_size_slider->Maximum(1.f);
    brush_size_slider->Minimum(0.01f);
    brush_size_slider->Interval(0.001f);
    brush_size_slider->ValueChanged += [&, this] (float val) mutable
    {
        float newval = val * 50;
        tool_size_label->Content(std::format("Size: {}", Math::Round(newval, 1)));
        BrushSizeChanged(newval);
    };
    brush_size_slider->CurrentValue(0.5f);
    brush_size_slider->SetClicked(false);


    brush_percent_slider = new Slider(right_row_layout);
    brush_percent_slider->Size({0, row_height, 1, 0});
    brush_percent_slider->Maximum(1.f);
    brush_percent_slider->Minimum(0.f);
    brush_percent_slider->Interval(0.001f);
    brush_percent_slider->ValueChanged += [&, this, tool_percent_label] (float val) mutable
    {
        float newval = val * 100.f;
        tool_percent_label->Content(std::format("Percent: {}%", Math::Floor(newval)));
        BrushPercentChanged(newval);
    };
    brush_percent_slider->CurrentValue(1.f);
    brush_percent_slider->SetClicked(false);


    auto* tile_sim_checkbox = new Checkbox(right_row_layout);
    tile_sim_checkbox->Size({row_height, row_height, 0, 0});
    tile_sim_checkbox->OnClickEvent += [&, this] (Vector2 _, JUI::MouseButton _2) mutable {
        tile_sim_disabled = !tile_sim_disabled;
        TileSimulationDisabledChanged.Invoke(tile_sim_disabled);

        // Set the visual state of the "step" buttonns
        step_btn->SetEnabled(tile_sim_disabled);
        step2_btn->SetEnabled(tile_sim_disabled);
        step3_btn->SetEnabled(tile_sim_disabled);
    };



    step_btn = new TextButton(left_row_layout);
    step_btn->Size({100_percent, UDim(row_height, 0)});
    step_btn->Content("Step");
    step_btn->BGColor(Colors::LightGray);
    step_btn->BaseBGColor(Colors::LightGray);
    step_btn->Disable();
    step_btn->OnClickEvent += [&, this] (auto a, auto b) mutable {
        TileSimulationStep.Invoke(1);
    };

    step2_btn = new TextButton(left_row_layout);
    step2_btn->Size({100_percent, UDim(row_height, 0)});
    step2_btn->Content("Step 10");
    step2_btn->BGColor(Colors::LightGray);
    step2_btn->BaseBGColor(Colors::LightGray);
    step2_btn->Disable();
    step2_btn->OnClickEvent += [&, this] (auto a, auto b) mutable {
        TileSimulationStep.Invoke(10);
    };

    step3_btn = new TextButton(left_row_layout);
    step3_btn->Size({100_percent, UDim(row_height, 0)});
    step3_btn->Content("Step 100");
    step3_btn->BGColor(Colors::LightGray);
    step3_btn->BaseBGColor(Colors::LightGray);
    step3_btn->Disable();
    step3_btn->OnClickEvent += [&, this] (auto a, auto b) mutable {
        TileSimulationStep.Invoke(100);
    };

    BrushSizeChanged += [this] (float value) mutable {
        brush_radius = value;
    };

    BrushPercentChanged += [this] (float value) mutable {
        brush_density = value;
    };


    Enable(WorldEditorEnabledByDefault);
}

void CaveGame::Client::TileTool::BrushRadius(float size) {
    brush_size_slider->CurrentValue(size/50.f);
    tool_size_label->Content(std::format("Size: {}", Math::Round(size, 1)));
    brush_radius = size;
}


void CaveGame::Client::TileTool::Enable(bool value) {
    this->enabled = value;

    CaveGame::Logs::Info("Tile Editor Tool Toggled");

    this->Visible(enabled);
}

float CaveGame::Client::TileTool::BrushRadius() { return brush_radius;}

void CaveGame::Client::TileTool::BrushDensity(float percent) {
    brush_percent_slider->CurrentValue(percent / 100.f);
    tool_size_label->Content(std::format("Density: {}", Math::Round(percent, 1)));
    brush_density = percent;
}

float CaveGame::Client::TileTool::BrushDensity() const { return brush_density;}
