#pragma once

#include <Color4.hpp>
#include <set>
#include <string>
#include <J3ML/Math.hpp>
#include <map>
#include <vector>
#include <optional>
//#include <Core/Registry.hpp>

namespace CaveGame::Core
{

    /// Implementation of items
    /// 1. Need to create a TileItem associated with each tile, automatically.
    /// 2. Not a strict requirement, but might like to be able to read item data from a data file.
    /// This would remove the need to re-compile the entire program when items are changed.
    /// 3. Each item in-game should be a single instance of the Item class.
    /// 4. Items are represented via ItemStacks and ItemStack entities, which represent quantities of items
    /// in a container, or dropped on the ground.
    /// 5. Support Item Animated Sprites
    /// 6. Support lots of unique item abilities and effects.
    /// 7. Support attaching custom metadata to certain items.


    /// This enum defines combinatorial category flags to items to enable smart sorting.
    enum class ItemCategory
    {
        ANY, TILE_ITEM, ORE, TOOL, WEAPON, ARMOR, CLOTHING, FOOD, POTION, INGREDIENT, MEME, PLANT, METAL, ORGANIC, BUILDING_BLOCK,
        FLUID, SOIL, GAS, LIGHT_SOURCE, CRAFTING_STATION,
    };


    /// This enum defines a mutually-exclusive (one-at-a-time) 'Modifier' that items may randomly spawn with.
    /// It will affect item stats in a variety of ways, and some will even have special coding.
    enum class ItemModifiers {

        // These represent low-quality, crappy items, often with comical effects.
        FLIMSY, RUSTY, BROKEN, BENDY, RUBBERY, FRAGILE, FILTHY, DRAB, DULL, SCARRED, BURNED,

        // These primarily make an item more valuable.
        FABULOUS, GOLD_ENGRAVED, LOVELY, MAGNIFICENT,
        SPARKLING, PLAIN,
        // These primarily modify an item's visual and mechanical 'size'.
        TINY, LONG, GREATER,

        // These primarily modify an item's damage.
        SHARP, COMBATIVE,

        // These primarily modify an item's durability.
        HARDENED, FORGED, BRITTLE,  TOUGH,

        // These primarily affect an item's speed.
        LAZY, FRANTIC, LIGHT, HEAVY,

        // These primarily represent enchanted or magically imbued items.
        ELDRITCH, ARCANE, MYSTICAL, MYTHICAL, MAJESTIC,

        // These represent cursed or hexed items that have interesting drawbacks.
        ACCURSED, EVIL, MENACING, BLOODY, UGLY, FIERCE, NAUGHTY, DASTARDLY,

        // As-of-yet uncategorized modifiers.
        ANCIENT, VICTORIOUS, CONDEMNED,  CLEAN, LUCKY, POWERFUL, SUPER,
    };

    /*class ItemComponent
    {

    };

    class UseCooldown {};
    class Consumable {};
    class Drinkable {};
    class Eatable {};*/

    struct Item
    {
        std::string mnemonic;
        std::string display_name;
        std::string tooltip;
        std::string author;
        int max_stack;
        int value;
        std::vector<std::string> tags;
        std::vector<std::string> aliases;

        std::optional<Color4> sprite_color;

    protected:
    private:
    };


    class ItemFilter
    {

        std::set<ItemCategory> AllowedCategories;
        std::set<ItemCategory> BannedCategories;

        std::set<std::string> AllowedItems;
        std::set<std::string> BannedItems;
    };

    namespace StatusEffects {

    }

    /*
    namespace Items {

        static const Item Straw;

        //static const Item Gel {"gel", "Gel", };

        static const Item Glowstick;
        //static const Item RoastChicken {"roast-chicken", "Roast Chicken"};
        static const Item PumpkinPie;
        static const Item ShepherdsPie;

        static const Item CopperBar;
        static const Item IronBar;
        static const Item SilverBar;
        static const Item TungstenBar;
        static const Item PlatinumBar;
        static const Item GoldBar;
        static const Item LeadBar;

        static const Item ZiggyStardust;
        static const Item Nimdoc;
        static const Item OccupiedDwellinator;
        static const Item SawnoffShotty;

#pragma region Tools

#pragma endregion

#pragma region Weapons
        static const Item GutsyBlade;
#pragma region Daggers
    static const Item CopperDagger;
        static const Item IronDagger;
        static const Item TinDagger;
        static const Item SilverDagger;
#pragma endregion
#pragma endregion

#pragma region SciFi
        static const Item Shrinkray;
        static const Item PortalGun;
        static const Item GravityGun;
        static const Item TauCannon;
        static const Item PhotonSabre;

#pragma endregion

#pragma region Armor

#pragma endregion
#pragma region Accessories
    static const Item KittenEars;
        static const Item WolfEars;
#pragma region
#pragma region Accessories

#pragma endregion
#pragma endregion

#pragma region Foodstuff

        static const Item Wheat;
        static const Item Rice;

#pragma region Fruit
        static const Item Apple;
        static const Item Orange;
        static const Item Lemon;
        static const Item Carrot;
        static const Item Potato;
        static const Item Onion;
        static const Item Banana;
        static const Item MelonSlice;
        static const Item Tomato;
#pragma endregion
#pragma region Vegetables

#pragma endregion
#pragma region Cooked Dishes

#pragma endregion

#pragma region Soups

#pragma region Drinks
    static const Item HotCocoa;
        static const Item AppleCider;
        static const Item MelonJuice;
#pragma endregion
#pragma region Potions
        static const Item FeatherfallPotion;
        static const Item HP100Potion;
        static const Item HP250Potion;
        static const Item HP500Potion;
        static const Item HP2XPotion;
        static const Item HP5XPotion;
        static const Item HP10XPotion;

#pragma endregion


#pragma region Bottles
        static const Item EmptyBottle;
        static const Item WaterBottle;
        static const Item HoneyBottle;
        static const Item IceBottle;
        static const Item OilBottle;
        static const Item MilkBottle;
        static const Item SludgeBottle;
        static const Item EctoplasmBottle;
#pragma endregion

#pragma region Buckets
        static const Item EmptyBucket;
        static const Item WaterBucket;
        static const Item IceBucket;
        static const Item LavaBucket;
        static const Item MilkBucket;
        static const Item OilBucket;
        static const Item SludgeBucket;
        static const Item EctoplasmBucket;
#pragma endregion

    }*/


}

