#pragma once
#include <J3ML/LinearAlgebra/Vector2.hpp>
#include <JGL/JGL.h>

namespace CaveGame::Core
{
    // TODO: Rect2D class that is distinct from AABB2D

    struct Frame {
        Vector2 offset;
        Vector2 size;
        Vector2 origin;

        //AABB2D quad;
    };

    struct Rect2D {
        Vector2 position;
        Vector2 size;
    };

    struct Keyframe {
        Vector2 transform_scale;
        Vector2 transform_offset;


        Frame frame;
        float time;
    };

    struct Animation {
        std::vector<Keyframe> frames;
    };


    class Sprite {
    public:
        /// The default constructor does not initialize any members.
        Sprite() { }

        /// Constructs a not-animated sprite from a source texture, which assumes.
        Sprite(JGL::Texture* source);
        Sprite(JGL::Texture* source, int width, int height);
        Sprite(JGL::Texture* source, Vector2i size);
        Sprite(JGL::Texture* source, int offset_x, int offset_y, int width, int height);
        Sprite(JGL::Texture* source, Vector2i offset, Vector2i size);
        void AddFrame(const std::string& frame_id, const Frame& frame);
        void SetCurrentFrame();
        void DrawFrame();
        void DrawCurrentFrame();

        //void PlayAnimation(const KeyframeAnimation& anim);
    protected:
        JGL::Texture* source;
        std::vector<Frame> frames;
        std::vector<Animation> animations;
    private:
    };
}
