#pragma once
#include "Singleton.hpp"
#include <string>
#include <cstdint>
#include <vector>
#include "Color4.hpp"
#include "JJX/JSON.hpp"
#include <optional>
#include <functional>
#include <array>
#include <Core/Tile.hpp>
#include <J3ML/J3ML.hpp>
#include <map>
#include <fstream>
#include <filesystem>

namespace CaveGame::Core {
    //class ITileMap;

    //using ColorPallet = std::vector<Color4>;
    //using TileTiccFunc = std::function<void(const Tile& data, ITileMap* world, int x, int y)>;

    /*std::unordered_map<std::string, TileTiccFunc> tile_functions {
        {"check-spread", [](const Tile& data, ITileMap* world, int x, int y){

        }},
        {"check-decay"},
        {"check-decay-spread"},
        {"sand-ticc"},
        {"water-ticc"},
    };*/

    //using TileID = uint16_t;

    /// A Singleton that stores tile data.
    class TileRegistry : public Singleton<TileRegistry> {

    public:
        static constexpr int MaxTiles = 65536;

        ///
        void Register(Tile data);

        const Tile &Get(const std::string &mnemonic);

        const Tile &Get(u16 id);

        // TODO: Why convert to vector?
        std::vector<Tile> GetTileList();

        const std::array<Tile, MaxTiles>& GetTileArray();
        bool Exists(const std::string& mnemonic);
        bool Exists(u16 id);
        const Tile& GetByMnemonicID(const std::string& mnemonic);
        const Tile& GetByNumericID(uint16_t ID);

        uint16_t MnemonicToNumeric(const std::string& mnemonic);
        std::string NumericToMnemonic(uint16_t ID);
        const Tile& operator[](const std::string& mnemonic);
        const Tile& operator[](u16 ID);


    protected:

        void Register(TileID ID, const Tile &data);

    protected:

        std::array<bool, MaxTiles> consumed_ids{};
        std::array<Tile, MaxTiles> registered_tiles{};
        std::unordered_map<std::string, u16> name_to_id_mapping{};
        std::unordered_map<u16, std::string> id_to_name_mapping{};
        u16 current_index = 0;
    private:
    };

    /// Convenient access to the tile registry.
    static TileRegistry& Tiles() { return TileRegistry::Instance();}

    bool LoadTileMetadata(const std::filesystem::path& path);

    bool LoadTileMetadata();
}
