#include <Core/ItemRegistry.hpp>
#include <Core/Macros.hpp>
#include <filesystem>
#include <JJX/JSON.hpp>
#include <Core/JsonConversions.hpp>
#include "Core/Loggers.hpp"

namespace CaveGame::Core
{

    bool LoadItemMetadata(const std::filesystem::path &path) {
        using namespace JJX;
        std::string content = read_file(path);

        auto [data, parse_error] = json::parse(content);

        if (data.type != json::value_type::array) {
            // TODO: Error
            return false;
        }

        for (auto& entry : data.as_array()) {
            auto entry_obj = entry.as_object();
            Core::Item item;

            item.mnemonic = entry_obj["mnemonic-id"].string.value_or("");
            item.display_name = entry_obj["display-name"].string.value_or("");

            if (entry_obj.contains("tags"))
                item.tags = JsonConversions::parse_string_list(entry_obj["tags"]);


            // TODO: Support multiple overlaid "Sprites" with assigned colors.
            // TODO: Support animated sprites.
            std::string sprite_name;

            Items().Register(item);

        }

        return true;
    }

    bool LoadItemMetadata() {
        if (!LoadItemMetadata("assets/data/items.json")) return false;

        return true;
    }

    void ItemRegistry::Register(const Item& data) {

        Logs::Info(std::format("\tRegister Item: id:{} display_name:{} stack:{} value:{}", data.mnemonic, data.display_name, data.max_stack, data.value));

        registered_items.emplace(data.mnemonic, data);
    }

    bool ItemRegistry::Exists(const std::string &name) {
        return registered_items.contains(name);
    }

    const Item &ItemRegistry::Get(const std::string &name) {
        return registered_items.at(name);
    }

    std::unordered_map<std::string, Item> ItemRegistry::GetItemMap() { return registered_items;}

    const Item &ItemRegistry::operator[](const std::string &name) { return Get(name);}

}