#include <Core/Loggers.hpp>

namespace CaveGame::Logs {
    using namespace jlog;

    CaveGameLogger Info    {"CaveGame",            GlobalLogFile,Colors::Gray, Colors::Gray, Colors::Gray, Colors::Gray};
    CaveGameLogger Debug    {"CaveGame",            GlobalLogFile,Colors::Greens::Lime, Colors::Gray, Colors::Gray, Colors::Gray};
    CaveGameLogger Warning    {"CaveGame",            GlobalLogFile,Colors::Oranges::Gold, Colors::Gray, Colors::Gray, Colors::Gray};
    CaveGameLogger Error    {"CaveGame",            GlobalLogFile,Colors::Reds::Salmon, Colors::Gray, Colors::Gray, Colors::Gray};
    CaveGameLogger Fatal {"CaveGame", GlobalLogFile, Colors::Reds::DarkRed, Colors::Reds::DarkRed, Colors::Reds::DarkRed, Colors::Reds::DarkRed};

    CaveGameLogger Client    {"CaveGame::Client",    GlobalLogFile,Colors::Greens::Lime, Colors::Gray, Colors::Gray, Colors::Gray};
    CaveGameLogger Server    {"CaveGame::Server",    GlobalLogFile,Colors::Greens::Lime, Colors::Gray, Colors::Gray, Colors::Gray};
    CaveGameLogger Generator {"CaveGame::Generator", GlobalLogFile,Colors::Greens::Lime, Colors::Gray, Colors::Gray, Colors::Gray};
    CaveGameLogger Lighting  {"CaveGame::Generator", GlobalLogFile,Colors::Greens::Lime, Colors::Gray, Colors::Gray, Colors::Gray};

    void SetAllEnableConsole(bool log_to_console) {
        Info.EnableConsole(log_to_console);
        Debug.EnableConsole(log_to_console);
        Warning.EnableConsole(log_to_console);
        Error.EnableConsole(log_to_console);
        Fatal.EnableConsole(log_to_console);

        Client.EnableConsole(log_to_console);
        Server.EnableConsole(log_to_console);
        Generator.EnableConsole(log_to_console);
        Lighting.EnableConsole(log_to_console);
    }

    void SetAllEnableFile(bool log_to_file) {
        Info.EnableFile(log_to_file);
        Debug.EnableFile(log_to_file);
        Warning.EnableFile(log_to_file);
        Error.EnableFile(log_to_file);
        Fatal.EnableFile(log_to_file);

        Client.EnableFile(log_to_file);
        Server.EnableFile(log_to_file);
        Generator.EnableFile(log_to_file);
        Lighting.EnableFile(log_to_file);
    }

    void EnableAllToConsole() {
        SetAllEnableConsole(true);
    }

    void EnableAllToFile() {
        SetAllEnableFile(true);
    }

    void DisableAllToConsole() {
        SetAllEnableConsole(false);
    }

    void DisableAllToFile() {
        SetAllEnableFile(false);
    }

    void EnableAll() {
        EnableAllToConsole();
        EnableAllToFile();
    }

    void DisableAll() {
        DisableAllToConsole();
        DisableAllToFile();
    }

    void SetIncludeSourceLocationAll(bool inc_location) {
        Info.IncludeLocation(inc_location);
        Debug.IncludeLocation(inc_location);
        Warning.IncludeLocation(inc_location);
        Error.IncludeLocation(inc_location);
        Fatal.IncludeLocation(inc_location);

        Client.IncludeLocation(inc_location);
        Server.IncludeLocation(inc_location);
        Generator.IncludeLocation(inc_location);
        Lighting.IncludeLocation(inc_location);
    }

    void SetIncludeTimestampAll(bool inc_timestamp) {
        Info.IncludeTimestamp(inc_timestamp);
        Debug.IncludeTimestamp(inc_timestamp);
        Warning.IncludeTimestamp(inc_timestamp);
        Error.IncludeTimestamp(inc_timestamp);
        Fatal.IncludeTimestamp(inc_timestamp);

        Client.IncludeTimestamp(inc_timestamp);
        Server.IncludeTimestamp(inc_timestamp);
        Generator.IncludeTimestamp(inc_timestamp);
        Lighting.IncludeTimestamp(inc_timestamp);
    }

    void EnableSourceLocationAll() {
        SetIncludeSourceLocationAll(true);
    }

    void DisableSourceLocationAll() {
        SetIncludeSourceLocationAll(false);
    }

    void EnableTimestampAll() {
        SetIncludeTimestampAll(true);
    }

    void DisableTimestampAll() {
        SetIncludeTimestampAll(false);
    }

    // TODO: This is duplicated here and in ReCaveGame::Loggers.cpp
    // Bring into jlog as utility functions.
    std::string format_timestamp(Timestamp ts) {
        return std::format(
                "{}-{}-{} {}:{}:{}.{}",
                ts.Year(),
                ts.Month(),
                ts.Day(),
                ts.Hour().count(),
                ts.Minute().count(),
                ts.Second().count(),
                ts.Millisecond().count());
    }

    // TODO: This is duplicated here and in ReCaveGame::Loggers.cpp
    // Bring into jlog as utility functions.
    std::string format_source_location(const std::source_location &location) {
        return std::format("{} @ {}:{}", location.function_name(), location.file_name(), location.line());
    }

    GenericLogger Network   {"CaveGame::Generator", GlobalLogFile,Colors::Greens::Lime, Colors::Gray, Colors::Gray, Colors::Gray};

    CaveGameLogger::CaveGameLogger(const std::string &context, std::ofstream &file, Color4 contextColor,
                                   Color4 timestampColor, Color4 locationColor, Color4 pointerColor,
                                   Color4 messageColor)
            : GenericLogger(context, file, contextColor, timestampColor, locationColor, pointerColor, messageColor) { }

    void CaveGameLogger::Log(const std::string &message, const std::source_location &location, const Timestamp &ts) {
        GenericLogger::Log(message, location, ts);

        std::string formatted_timestamp = format_timestamp(ts);
        std::string formatted_source_location = format_source_location(location);


        std::string final_result = std::format("[{}] [{}] {}", formatted_timestamp, this->ConsoleLogger::context, message);

        OnLog.Invoke(final_result, messageColor);
    }
}